/**
 * 4399平台技术前端框架核心
 * @author  YJC, web.4399.com
 * @version  1.1
 * @since  2012-05-25
 */
var  core =
(function(W, D, undefined){
  var  nodes = { // 部分DOM节点的引用
      head: D.head || D.getElementsByTagName('head')[0] || D.documentElement
    },
    // 当前正在添加的script/状态为interactive的script，用于IE6-9
    current_script, interactive_script,
    factory_cache, // 匿名模块缓存
    event_cache, // 事件缓存
    event_id, // 当前事件集ID
    running = false, // 标记是否正在执行run()
    event_set = [], // 事件集合
    js = [], // 等待加载的js
    fn = [], // 等待执行的函数
    module = [], // 模块集合
    config = { // 全局配置
      CHARSET: 'utf-8' // 默认js字符编码
      // DOMAIN // （cookie）默认域
      // PATH // （cookie）默认路径
      // DIR_JS // 默认js路径
      // MAIN_VERSION // 主体模块版本
    },

    util = { // 工具箱
      /**
       * 单个js文件加载器
       */
      js_loader: (function(){
        // 加载队列
        var queue = [];
        // 清除指定的script及其绑定的事件
        function clear(i) {
          queue[i].script.onload = queue[i].script.onreadystatechange = queue[i].script.onerror = null;
          queue[i].script.src = '';
          queue[i].script.parentNode.removeChild(queue[i].script);
          queue[i].script = null;
          return delete queue[i];
        }
        // 开始加载并执行指定的js
        function exec(index, src, charset){
          if ( ! (index in queue)) return false;
          queue[index].script.charset = charset || config.CHARSET;
          queue[index].script.src = src;
          current_script = queue[index].script;
          nodes.head.insertBefore(queue[index].script, nodes.head.firstChild);
          current_script = undefined;
          return true;
        }
        return {
          /**
           * 动态加载Javascript文件
           * @param {string} js_file 可解析的js文件路径
           * @param {function} callback 载入js后的回调函数
           * @param {number} time 载入js的延时秒数
           * @param {string} charset js文件的字符编码
           * @return {boolean|number} 当前操作在加载列表的索引值
           */
          load: function(js_file, callback, time, charset){
            if ( ! js_file) return false;
            if (typeof callback == 'number') time = callback;
            var i = queue.length;
            queue[i] = { script: D.createElement('script') };
            queue[i].script.onload = queue[i].script.onreadystatechange = queue[i].script.onerror = function(){
              if (queue[i].script.readyState
                && queue[i].script.readyState != 'loaded' && queue[i].script.readyState != 'complete') return;
              clear(i); // 加载完成后清除无用的引用
              if (typeof callback == 'function') callback();
            };
            if (isNaN(time) || time == 0) {
              exec(i, js_file, charset);
              return i;
            }
            queue[i].load = setTimeout(
              function(){exec(i, js_file, charset);},
              time * 1000
            );
            return i;
          },
          /**
           * 取消加载指定的js
           * @param {number} index js在加载队列的索引值
           */
          abort: function(index){
            if ( ! (index in queue)) return false;
            clearTimeout(queue[index].load);
            return clear(index);
          }
        };
      })(),
      /**
       * IE6-9下，在外部js文件执行过程中获取引用了此文件的script标签
       * 此函数取自 seaJS v1.1.0, http://seajs.org/
       */
      get_current_script: function(){
        if (current_script) return current_script;
        if (interactive_script &&
          interactive_script.readyState === 'interactive') {
          return interactive_script;
        }

        var script, scripts = nodes.head.getElementsByTagName('script');

        for (var i = 0; i < scripts.length; i++) {
          script = scripts[i];
          if (script.readyState === 'interactive') {
            interactive_script = script;
            return interactive_script;
          }
        }
      },
      /**
       * 获取url的绝对路径
       * @param {string} url 目标路径
       * @param {string} base 基准路径
       * @return {string} 目标绝对路径
       */
      absolute_url: function(url, base){
        var i, href, root;
        base = base || W.location.href;
        i = base.indexOf("://");
        if (i == -1) return false; // 基准路径有误
        i = base.indexOf("/", i + 3);
        if (i == -1) {
          base += '/';
          root = base;
        }
        else root = base.substr(0, i + 1);
        if (url.indexOf('http://') === 0 || url.indexOf('https://') === 0) {
          href = url;
        } else if (url.charAt(0) == '/') {
          href = root + url.substr(1);
        } else {
          i = base.indexOf('?');
          if (i != -1) base = base.substring(0, i);
          i = base.indexOf('#');
          if (i != -1) base = base.substring(0, i);
          href = base.substr(0, base.lastIndexOf('/') + 1) + url;
        }
        var p = href.split('/'), u = [];
        for (i in p) {
          if (p[i] == '.' || (p[i] == '' && i > 1 && i < p.length - 1)) continue;
          if (p[i] == '..') {
            if (u.length > 3) u.pop();
            else continue;
          }
          else u.push(p[i]);
        }
        href = u.join('/');
        i = href.indexOf('?');
        if (i != -1) href = href.substring(0, i);
        i = href.indexOf('#');
        return i == -1? href: href.substring(0, i);
      },
      /**
       * 计数器
       */
      counter: (function(){
        var i = 0;
        return function(){ return ++i; };
      })(),
      /**
       * 获取唯一ID
       */
      unique_id: (function(){
        var  i = 0;
        return function(){
          var  id = "ID-" + (i++);
          if (D.getElementById(id)) return arguments.callee();
          return id;
        };
      })(),
      /**
       * 获取body节点
       */
      get_body: function(){
        if ( ! nodes.body) nodes.body = D.body || D.getElementsByTagName('body')[0] || D.documentElement;
        return nodes.body;
      },
      /**
       * 获取title节点
       */
      get_title: function(){
        if ( ! nodes.title) nodes.title = D.getElementsByTagName('title')[0];
        return nodes.title;
      },
      /**
       * 将模块标识解析为绝对路径的url
       * @param {string} index 模块标识
       * @return {boolean|string} url
       */
      parse_index: function(index){
        if ( ! index || typeof index != 'string') return false;
        var base, i = index.indexOf('?');
        if (i != -1) {
          index = index.substr(0, i);
        }
        i = (index.indexOf('://') == -1);
        // 当并非绝对路径且不以.js结尾的，补全结尾的.js
        if (i && index.substr(index.length - 3).toLowerCase() != '.js') {
          index += '.js';
        }
        // 当并非绝对路径且不以.或/开头，则以全局变量DIR_JS或DIRS[#]为基准路径
        if (i && index.charAt(0) != '.' && index.charAt(0) != '/') {
          i = index.indexOf('#');
          if (i == -1 && config['DIR_JS']) {
            base = config['DIR_JS'];
          } else if (i > 0 && ('DIRS' in config)) {
            var j = index.substr(0, i);
            if (config['DIRS'][j]) {
              index = index.substr(i + 1);
              base = util.absolute_url(config['DIRS'][j]);
            }
          }
        }
        // 默认以当前路径为基准路径
        return util.absolute_url(index, base || W.location.href);
      },
      /**
       * 深度复制对象
       */
      clone: function(object) {
        if (typeof object != 'object') return {};
        var i, o = (object instanceof Array)? []: {};
        for (i in object) {
          if (typeof object[i] == 'object') o[i] = arguments.callee(object[i]);
          else o[i] = object[i];
        }
        return o;
      },
      /**
       * 用一个对象扩展对象，此函数会改变 source_object
       * 如果希望 source_object 不变，这样使用 util.extend(util.clone(source_object), extend_object)
       */
      extend: function(source_object, extend_object) {
        if (typeof source_object != 'object') source_object = {};
        if (typeof extend_object != 'object') return source_object;
        var i;
        for (i in extend_object) {
          if (typeof extend_object[i] == 'object') {
            source_object[i] = util.clone(extend_object[i]);
          } else {
            source_object[i] = extend_object[i];
          }
        }
        return source_object;
      },
      core_data: function(name){
        if ( ! ('core' in nodes)) return false;
        return nodes.core.getAttribute('data-' + name) || '';
      }
    };

  // 执行函数队列中满足依赖关系的函数
  function run() {
    if (running) return;
    running = true;
    var i, j, valid;
    for (i in js) {
      if (i in module) load(i, '', 0, module[i].charset);
    }
    for (i in fn) {
      valid = true;
      for (j in fn[i][0]) {
        if ( ! (fn[i][0][j] in module)) {
          fn[i][0][j] = util.parse_index(fn[i][0][j]);
        }
        if ( ! module[fn[i][0][j]].loaded) {
          valid = false;
          break;
        }
      }
      if (valid) {
        fn[i][0] = null;
        fn[i][1]();
        fn[i][1] = null;
        delete fn[i];
      }
    }
    running = false;
  }
  /**
   * 载入一个js文件
   * @param {string} file 可解析的js文件路径/库文件的索引
   * @param {string|Array} deps 依赖关系
   * @param {number} timeout 延时加载秒数
   * @param {string} charset js文件的字符编码
   */
  function load(file, deps, timeout, charset) {
    file = extend(file, '', deps, charset);
    if (file in module) {
      if (module[file].loading) return;
      if (module[file].loaded) {
        run();
        return;
      }
    } else return;
    deps = module[file].deps;
    var i, valid = true;
    if (deps.length) { // 依赖关系检查
      for (i in deps) {
        if ( ! deps[i]) continue;
        deps[i] = extend(deps[i]);
        if ( ! module[deps[i]].loaded) {
          if ( ! module[deps[i]].loading) {
            // 载入未加载的依赖文件
            js[deps[i]] = 1;
            load(deps[i], '', timeout, module[deps[i]].charset);
          }
          valid = false;
        }
      }
    }
    if (valid) {
      module[file].loading = true;
      util.js_loader.load(
        file + (module[file].version? '?' + module[file].version: ''),
        function(){
          if (factory_cache) {
            module[file].factory = factory_cache;
            factory_cache = undefined;
          }
          if (event_cache) {
            module[file].event = event_cache;
            event_cache = undefined;
          }
          event_id = undefined;
          module[file].loaded = true;
          module[file].loading = false;
          if (file in js) delete js[file];
          run();
        },
        timeout,
        charset
      );
    }
  }

  /**
   * 扩展模块库定义
   * @param {string} file 模块标识
   * @param {string} version 版本
   * @param {string|Array} deps 依赖关系
   * @param {string} charset 编码
   * @return {string} 解析后的模块标识
   */
  function extend(file, version, deps, charset) {
    var index = util.parse_index(file);
    if ( ! index) throw "[CORE ERROR #1] 解析模块路径失败 : " + file;
    if (index in module) return index;
    if ( ! (deps instanceof Array)) {
      deps = (deps && typeof deps == 'string')? deps.split(' '): [];
    }
    var i = file.indexOf('?');
    if (i >= 0 && ! version) {
      version = file.substr(i + 1) || (new Date).getTime();
    }
    module[index] = {
      loaded: false,
      loading: false,
      deps: deps,
      version: version,
      charset: charset || config.CHARSET
    };
    return index;
  }
  return {
    debug: {module: module, js: js, fn: fn, event: event_set},
    nodes: nodes,
    util: util,
    event: (function(){
      function emit_event(e, args){
        if (args.length > 1) {
          var i, a = [];
          for (i = 1; i < args.length; i++) {
            a.push(args[i]);
          }
          return e.apply(null, a);
        } else {
          return e();
        }
      }
      function e(index) {
        index = util.parse_index(index);
        if ( ! (index in module)) throw "[CORE ERROR #23] 未定义的模块 : " + index;
        return {
          emit: function(name){
            return emit_event(module[index]['event'][name], arguments);
          }
        };
      }
      e.on = function(name, fn) {
        if (typeof fn != 'function') throw "[CORE ERROR #30] 声明事件必须传入函数";
        if ( ! event_id) event_id = util.counter();
        if ( ! (event_id in event_set)) event_set[event_id] = {};
        event_set[event_id][name] = fn;
        var i, script = util.get_current_script();
        if (script && script.src) {
          i = script.src.indexOf('?');
          i = (i == -1)? script.src: script.src.substr(0, i);
          if ( ! (i in module)) {
            throw "[CORE ERROR #22] 未定义的模块 : " + script.src;
          }
          if ( ! ('event' in module[i])) module[i].event = event_set[event_id];
        } else {
          if ( ! event_cache) event_cache = event_set[event_id];
        }
        return fn;
      };
      e.emit = function(name) {
        if ( ! event_id || ! (event_id in event_set) || ! (name in event_set[event_id])) {
          throw "[CORE ERROR #31] 调用了未声明的事件 " + event_id + ":" + name;
        }
        return emit_event(event_set[event_id][name], arguments);
      };
      return e;
    })(),
    define: function(factory){
      var i, script = util.get_current_script();
      if (arguments.length > 1) {
        var init = arguments[1];
        if (typeof factory == 'function') factory = factory();
        if ((typeof factory == 'object' || typeof factory == 'function') && typeof factory[init] == 'function') {
          if (arguments.length > 2) {
            var args = [];
            for (i = 2; i < arguments.length; i++) {
              args.push(arguments[i]);
            }
            factory[init].apply(null, args);
          }
          else factory[init]();
        }
      }
      if (script && script.src) {
        i = script.src.indexOf('?');
        i = (i == -1)? script.src: script.src.substr(0, i);
        if ( ! (i in module)) {
          if (typeof console === 'object' && console.log) {
            console.log("[CORE ERROR #20] 未定义的模块 : " + script.src);
          }
        }
        module[i].factory = factory;
      } else {
        factory_cache = factory;
      }
    },
    require: function(index){
      if (index.substr(index.length - 4).toLowerCase() == '.css') {
        var link = D.createElement('link');
        link.rel = "stylesheet";
        link.type = "text/css";
        link.href = index;
        nodes.head.insertBefore(link, nodes.head.firstChild);
        link = null;
      } else {
        index = util.parse_index(index);
        if ( ! (index in module) || ! module[index].factory) {
          try{throw "[CORE ERROR #21] 未定义的模块 : " + index;}catch(e){/*alert(e);*/}
        }
        return module[index].factory;
      }
    },
    async: function(files, callback, deps, timeout, charset){
      if ( ! files) return;
      var i, run_deps;
      files = files.split(' ');
      if (deps) {
        deps = deps.split(' ');
        run_deps = files.concat(deps);
      } else {
        deps = [];
        run_deps = files;
      }
      if (typeof callback == 'function') fn.push([run_deps, callback]);
      for (i in files) {
        js[util.parse_index(files[i])] = 1;
        load(files[i], deps, timeout, charset);
      }
    },
    extend: extend,
    set_config: function(c){
      if (typeof c != 'object') return config;
      var i, e, r;
      if ('DIR_JS' in c) {
        c['DIR_JS'] = util.absolute_url(c['DIR_JS']);
      }
      if ('modules' in c) {
        e = c['modules'];
        delete c['modules'];
      }
      if ('autorun' in c) {
        r = c['autorun'];
        delete c['autorun'];
      }
      util.extend(config, c);
      if (e) {
        for (i in e) {
          extend(e[i][0], e[i][1], e[i][2], e[i][3]);
        }
      }
      if (typeof r == 'function') r();
      return config;
    },
    clear_factory: function(index) {
      index = util.parse_index(index);
      if ( ! (index in module) || ! module[index].factory) return;
      module[index].factory = undefined;
    }
  };
})(window, document),

util  = core.util,
events  = core.event,
define  = core.define,
require  = core.require,
async  = core.async;

(function(W, D, C){
  var s = D.getElementsByTagName('script');
  s = s[s.length - 1];
  C.nodes.core = s;
  var c = s.getAttribute('data-config'),
    d = s.getAttribute('data-deps') || '',
    m = s.getAttribute('data-main'),
    end = s.getAttribute('data-end') || '';
  if (c) {
    C.async(c + '?', function(){
      W.CONFIG = C.set_config(C.require(c));
      C.clear_factory(c);
      if (m) {
        C.async(
          m + (('MAIN_VERSION' in W.CONFIG)? '?' + W.CONFIG['MAIN_VERSION']: ''),
          end? function(){ async(end); }: 0,
          d
        );
      }
    });
  } else if (m) {
    C.async(m, 0, d);
  }
})(window, document, core);
